#ifndef LowPassPrototypesH
#define LowPassPrototypesH

#include "Common.h"
#include "PFiftyOneRevD.h"
#include "LowPassRoots.h"

#define MAX_POLE_COUNT 20

typedef enum
{
  stMax,
  stMin
} TOurSortTypes;

// This MUST be at least 2*MAX_POLE_COUNT because some
// filter polys are defined in terms of 2 * NumPoles
#define ARRAY_DIM 50

// These coeff form H(s) = (N2*s^2 + N1*s + N0) / (D2*s^2 + D1*s + D0)
// NumSections is the number of 1st and 2nd order polynomial factors .
typedef struct
{
  double N2[ARRAY_DIM];
  double N1[ARRAY_DIM];
  double N0[ARRAY_DIM];
  double D2[ARRAY_DIM];
  double D1[ARRAY_DIM];
  double D0[ARRAY_DIM];
  int NumSections;
} TSPlaneCoeff;

// These are the available filter polynomials. NOT_IIR is for code testing.
typedef enum
{
  BUTTERWORTH,
  GAUSSIAN,
  BESSEL,
  ADJUSTABLE,
  CHEBYSHEV,
  INVERSE_CHEBY,
  PAPOULIS,
  ELLIPTIC,
  NOT_IIR
} TFilterPoly;

// This structure defines the low pass filter prototype.
// The 3 dB corner frequency is 1 rad/sec for all filters.
typedef struct
{
  TFilterPoly ProtoType;  // Butterworth, Cheby, etc.
  int NumPoles;      // Pole count
  double Ripple;     // Passband Ripple for the Elliptic and Chebyshev
  double StopBanddB; // Stop Band Attenuation in dB for the Elliptic and Inverse Cheby
  double Gamma; // Controls the transition bandwidth on the Adjustable Gauss. -1 <= Gamma <= 1
} TLowPassParams;


TSPlaneCoeff CalcLowPassProtoCoeff( TLowPassParams Filt );

void SetCornerFreq(
	int Count,
	double *D2,
	double *D1,
	double *D0,
	double *N2,
	double *N1,
	double *N0 );

int GetFilterCoeff(
	int RootCount,
	complex double *Roots,
	double *A2,
	double *A1,
	double *A0 );

int RebuildPoly(
	int PolyCount,
	double *PolyCoeff,
	double *A2,
	double *A1,
	double *A0 );

void SortRootsByZeta(
	complex double *Roots,
	int Count,
	TOurSortTypes SortType );

bool HeapIndexSort(
	double *Data,
	int *Index, int N,
	TOurSortTypes SortType );

#endif

